﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DTO;
using Entity;
using Shared;
using Dto2Entities;
using static System.Net.Mime.MediaTypeNames;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace ServicesApi
{
    public class SourceService : ISourceService<SourceDTO>
    {
        private ISourceService<Source> srcService;

        public SourceService(ISourceService<Source> src)
        {
            srcService = src;
        }
        public async Task AddSource(SourceDTO source)
        {
            await srcService.AddSource(source.ToEntity());
        }

        public async Task<PaginationResult<SourceDTO>> GetAll()
        {
            var sources = srcService.GetAll().Result.items;
            return new PaginationResult<SourceDTO>(sources.Count(), 0, 10, sources.ToDto());
        }

        public async Task<int> GetLastSourceId()
        {
            return await srcService.GetLastSourceId();
        }

        public async Task<PaginationResult<SourceDTO>> GetSomesSource(int page, int count)
        {
            var sources = (await srcService.GetSomesSource(page, count)).items;
            return new PaginationResult<SourceDTO>(sources.Count(), page, count, sources.ToDto());
        }

        public async Task<PaginationResult<SourceDTO>> GetSourceByDate(int date)
        {
            var sources = (await  srcService.GetSourceByDate(date)).items;
            return new PaginationResult<SourceDTO>(sources.Count(), 0, 10, sources.ToDto());
        }

        public async Task<SourceDTO> GetSourceById(int id)
        {
            return (await srcService.GetSourceById(id)).ToDto();
        }

        public async Task<SourceDTO> GetSourceByTitle(string title)
        {
            return srcService.GetSourceByTitle(title).Result.ToDto();
        }

        public async Task<SourceDTO> GetSourceByType(int type)
        {
            return srcService.GetSourceByType(type).Result.ToDto();
        }

        public async Task RemoveSource(int id)
        {
            await srcService.RemoveSource(id);
        }

        public async Task UpdateSource(int id, SourceDTO source)
        {
            await srcService.UpdateSource(id, source.ToEntity());
        }
    }
}
